"use strict";
/*
 * Copyright 2025 Palantir Technologies, Inc. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.RadioGroup = void 0;
const tslib_1 = require("tslib");
const classnames_1 = tslib_1.__importDefault(require("classnames"));
const React = tslib_1.__importStar(require("react"));
const common_1 = require("../../common");
const Errors = tslib_1.__importStar(require("../../common/errors"));
const utils_1 = require("../../common/utils");
const useValidateProps_1 = require("../../hooks/useValidateProps");
const radioCard_1 = require("../control-card/radioCard");
const controls_1 = require("./controls");
/**
 * Radio group component.
 *
 * @see https://blueprintjs.com/docs/#core/components/radio.radiogroup
 */
const RadioGroup = props => {
    const { children, className, disabled, inline, label, name, onChange, options, selectedValue, ...htmlProps } = props;
    // a unique name for this group, which can be overridden by `name` prop.
    const autoGroupName = React.useMemo(() => nextName(), []);
    const labelId = React.useMemo(() => (0, utils_1.uniqueId)("label"), []);
    (0, useValidateProps_1.useValidateProps)(() => {
        if (children != null && options != null) {
            console.warn(Errors.RADIOGROUP_WARN_CHILDREN_OPTIONS_MUTEX);
        }
    }, [children, options]);
    const getRadioProps = React.useCallback((optionProps) => {
        const { className: optionClassName, disabled: optionDisabled, value } = optionProps;
        return {
            checked: value === selectedValue,
            className: optionClassName,
            disabled: optionDisabled || disabled,
            inline,
            name: name == null ? autoGroupName : name,
            onChange,
            value,
        };
    }, [autoGroupName, disabled, inline, name, onChange, selectedValue]);
    const renderChildren = () => {
        return React.Children.map(children, child => {
            if ((0, utils_1.isElementOfType)(child, controls_1.Radio) || (0, utils_1.isElementOfType)(child, radioCard_1.RadioCard)) {
                return React.cloneElement(
                // Need this cast here to suppress a TS error caused by differing `ref` types for the Radio and
                // RadioCard components. We aren't injecting a ref, so we don't need to be strict about that
                // incompatibility.
                child, getRadioProps(child.props));
            }
            return child;
        });
    };
    const renderOptions = () => {
        return options === null || options === void 0 ? void 0 : options.map(option => (React.createElement(controls_1.Radio, { ...getRadioProps(option), key: option.value, labelElement: option.label || option.value })));
    };
    return (React.createElement("div", { role: "radiogroup", "aria-labelledby": label ? labelId : undefined, ...(0, common_1.removeNonHTMLProps)(htmlProps), className: (0, classnames_1.default)(common_1.Classes.RADIO_GROUP, className) },
        label && (React.createElement("label", { className: common_1.Classes.LABEL, id: labelId }, label)),
        Array.isArray(options) ? renderOptions() : renderChildren()));
};
exports.RadioGroup = RadioGroup;
exports.RadioGroup.displayName = `${common_1.DISPLAYNAME_PREFIX}.RadioGroup`;
let counter = 0;
function nextName() {
    return `${exports.RadioGroup.displayName}-${counter++}`;
}
//# sourceMappingURL=radioGroup.js.map