"use strict";
/*
 * Copyright 2024 Palantir Technologies, Inc. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.Toast2 = exports.Toast = void 0;
const tslib_1 = require("tslib");
const classnames_1 = tslib_1.__importDefault(require("classnames"));
const React = tslib_1.__importStar(require("react"));
const icons_1 = require("@blueprintjs/icons");
const common_1 = require("../../common");
const props_1 = require("../../common/props");
const useTimeout_1 = require("../../hooks/useTimeout");
const buttonGroup_1 = require("../button/buttonGroup");
const buttons_1 = require("../button/buttons");
const icon_1 = require("../icon/icon");
/**
 * Toast component.
 *
 * @see https://blueprintjs.com/docs/#core/components/toast
 */
exports.Toast = React.forwardRef((props, ref) => {
    const { action, className, icon, intent, isCloseButtonShown = true, message, onDismiss, timeout = 5000 } = props;
    const [isTimeoutStarted, setIsTimeoutStarted] = React.useState(false);
    const startTimeout = React.useCallback(() => setIsTimeoutStarted(true), []);
    const clearTimeout = React.useCallback(() => setIsTimeoutStarted(false), []);
    // Per docs: "Providing a value less than or equal to 0 will disable the timeout (this is discouraged)."
    const isTimeoutEnabled = timeout != null && timeout > 0;
    // timeout is triggered & cancelled by updating `isTimeoutStarted` state
    (0, useTimeout_1.useTimeout)(() => {
        triggerDismiss(true);
    }, isTimeoutStarted && isTimeoutEnabled ? timeout : null);
    // start timeout on mount or change, cancel on unmount
    React.useEffect(() => {
        if (isTimeoutEnabled) {
            startTimeout();
        }
        else {
            clearTimeout();
        }
        return clearTimeout;
    }, [clearTimeout, startTimeout, isTimeoutEnabled, timeout]);
    const triggerDismiss = React.useCallback((didTimeoutExpire) => {
        clearTimeout();
        onDismiss === null || onDismiss === void 0 ? void 0 : onDismiss(didTimeoutExpire);
    }, [clearTimeout, onDismiss]);
    const handleCloseClick = React.useCallback(() => triggerDismiss(false), [triggerDismiss]);
    const handleActionClick = React.useCallback((e) => {
        var _a;
        (_a = action === null || action === void 0 ? void 0 : action.onClick) === null || _a === void 0 ? void 0 : _a.call(action, e);
        triggerDismiss(false);
    }, [action, triggerDismiss]);
    return (React.createElement("div", { className: (0, classnames_1.default)(common_1.Classes.TOAST, common_1.Classes.intentClass(intent), className), 
        // Pause timeouts if users are hovering over or click on the toast. The toast may have
        // actions the user wants to click. It'd be a poor experience to "pull the toast" out
        // from under them.
        onBlur: startTimeout, onFocus: clearTimeout, onMouseEnter: clearTimeout, onMouseLeave: startTimeout, ref: ref, tabIndex: 0 },
        React.createElement(icon_1.Icon, { icon: icon }),
        React.createElement("span", { className: common_1.Classes.TOAST_MESSAGE, role: "alert" }, message),
        React.createElement(buttonGroup_1.ButtonGroup, { variant: "minimal" },
            action && React.createElement(buttons_1.AnchorButton, { ...action, intent: undefined, onClick: handleActionClick }),
            isCloseButtonShown && React.createElement(buttons_1.Button, { "aria-label": "Close", icon: React.createElement(icons_1.Cross, null), onClick: handleCloseClick }))));
});
exports.Toast.displayName = `${props_1.DISPLAYNAME_PREFIX}.Toast`;
/** @deprecated Use `Toast` instead */
exports.Toast2 = exports.Toast;
//# sourceMappingURL=toast.js.map